import re
from datetime import datetime, timedelta
from django.db import models

def listvalue( fn ):
	def wrapper( cls, v ):
		if not type(v) == 'list':
			v = [v]
		return fn( cls, v )
	return wrapper

def singlevalue( fn ):
	def wrapper( cls, v ):
		if type(v) == 'list':
			v = v[0]
		return fn( cls, v )
	return wrapper
	
class SearchableQS( models.query.QuerySet ):
	
	def _apply( self, filter_name, value ):
		if hasattr( self, filter_name ):
			func = getattr( self, filter_name )
			return func( value )
		return self
		
	def search( self, **kwargs ):
		if kwargs:
			filter_name, value = kwargs.popitem()
			if isinstance( value, str ) or isinstance( value, unicode ):
				try:
					value = eval( value )
				except:
					pass
			if kwargs: #If we still have params left in the search dict, keep filtering.
				return self._apply( filter_name, value ).search( **kwargs )			
			return self._apply( filter_name, value )
		
		return self

class SearchableManager( models.Manager ):
	
	def __init__( self, klass, *args, **kwargs ):
		super(SearchableManager, self).__init__(*args, **kwargs)
		self.klass = klass
	
	def get_query_set( self ):
		return self.klass( self.model, using=self._db )
	
	def search( self, **kwargs ):
		return self.get_query_set().search( **kwargs )
	

class SearchablePerson( SearchableQS ):

	@singlevalue
	def max_age( self, v ):
		today = datetime.now()
		born = today - timedelta( v*365.25 )
		return self.filter( date_of_birth__gte=born )

	@singlevalue
	def min_age( self, v ):
		today = datetime.now()
		born = today - timedelta( v*365.25 )
		return self.filter( date_of_birth__lte=born )

	@listvalue
	def postcode_in( self, v ):
		return self.filter( postcode__in=v )

	@listvalue
	def gender_in( self, v ):
		return self.filter( sex__in=v )

	def exclude_blocked( self, v ):
		return self.exclude( blocked=True )	

	@singlevalue
	def name( self, v ):
		return self.filter( name__istartswith=v )
		
		


from django.views.generic import View, ListView

from .models import Person

class PersonList( ListView ):	
	model = Person
	paginate_by = 25

	def get_context_data( self, *args, **kwargs ):
		ctx = super(PersonList, self).get_context_data(*args, **kwargs)
		query = self.request.GET.copy()
		query.pop( 'page', 0 )
		ctx['search_query'] = query.urlencode()
		return ctx
		
	def get_queryset( self, *args, **kwargs ):
		return Person.objects.search( **self.request.GET.dict() )


#encoding: utf-8

import math
from datetime import datetime

from django.db import models

from .searchable import SearchablePerson, SearchableManager
from .utils import parse_line_en

SEX_CHOICES = (
	(1,'Karl'), 
	(2,'Kona'), 
	(3,'Drengur'), 
	(4,'Stúlka')
)

MARITAL_CHOICES = (
	('1','Ógift'),
	('3', 'Gift eða staðfest samvist'),
	('4', 'Ekkill / Ekkja'),
	('5', 'Skilin að borð og sæng'),
	('6', 'Skilin að lögum'),
	('7', 'Hjón ekki í samvistum'),
	('8', 'Íslendingur í hjúskap með útlending'),
	('9', 'Hjúskaparstaða óupplýst'),
	('0', 'Íslendingur búsettur erlendis í hjúskap með útlendingi'),
	('L', 'Íslendingur með lögheimili á Íslandi í hjúskap með óskráðum útlendingi')
)

class Person( models.Model ):
	"""docstring for Person"""

	name = models.CharField( db_index=True, max_length=31 )			#Nafn
	sort_code = models.CharField( max_length=31 )		#Röðunarsvæði fyrir nafn

	personal_id = models.CharField( max_length=8 )		#Nafnnúmer
	ssn = models.CharField( db_index=True, max_length=10 )				#Kennitala
	ssn_spouse = models.CharField( max_length=10 )		#Kennitala maka
	ssn_guardian = models.CharField( max_length=10 )	#Kennitala umboðsmanns
	family_id = models.CharField( max_length=10 )		#Fjölskyldunúmer

	sex = models.CharField( max_length=1, choices=SEX_CHOICES )	#Kyn
	marital_status = models.CharField( max_length=1, choices=MARITAL_CHOICES) #Hjúskaparstaða

	domicile = models.CharField( max_length=12 )		#Lögheimiliskóði
	domicile_112 = models.CharField( max_length=12 )	#Lögheimiliskóði 1. Des s.l.
	domicile_last = models.CharField( max_length=12 )	#Síðasta lögheimili á Íslandi
	residence = models.CharField( max_length=12 )		#Aðsetur - lögheimiliskóði
	address = models.CharField( max_length=21 )			#Heimilisfang í nefnifalli
	address_thf = models.CharField( max_length=21 )		#Heimilisfang í þágufalli
			
	postcode = models.CharField( max_length=3 )			#Póstnúmer
			
	nationality = models.CharField( max_length=2 )		#Ríkisfang
	place_of_birth = models.CharField( max_length=4 )	#Fæðingastaður
	date_of_birth = models.DateField( )					#Fæðingadagur

	status = models.CharField( max_length=4 )			#Afdrif
	status_date = models.DateField( blank=True, null=True ) #Dagsetning afdrifa
	
	parish = models.CharField( max_length=3)			#Sókn
	blocked = models.BooleanField( default=False )		#Bannmerking
	mod_date = models.DateField( blank=True, null=True ) #Dagsetning hreyfingar
	
	objects = SearchableManager( SearchablePerson )
	
	def __unicode__( self ):
		return u'%s (%s)'%(self.name, self.ssn)

	@property
	def spouse( self ):
		if self.ssn_spouse:
			return Person.objects.get( ssn=self.ssn_spouse )
		return None
		
	@property
	def guardian( self ):
		if self.ssn_guardian:
			return Person.objects.get( ssn=self.ssn_guardian )
		return None
		
	@classmethod
	def update_from_string( self, s ):
		"""Parses an individual person from a national registry row"""
		info = parse_line_en( s )
		created = False
		try:
			p = Person.objects.get( ssn=info['ssn'] )
		except Person.DoesNotExist:
			p = Person( )
			fresh = True

		for k, v in info.items():
			if hasattr( p, k ):
				setattr( p, k, v )
		return p, fresh
	
	@classmethod
	def from_string( self, s ):
		"""Parses an individual person from a national registry row"""
		info = parse_line_en( s )
		p = Person( )
		fresh = True
		for k, v in info.items():
			if hasattr( p, k ):
				setattr( p, k, v )
		return p, fresh
	
	@property
	def age( self ):
		now = datetime.now()
		delta = now.date() - self.date_of_birth
		return int( math.floor( delta.days/365.25 ) )




#encoding: utf-8
from setuptools import setup, find_packages

setup(
	name = "django-thjodskra",
    version = "0.32",
    description = "django-thjodskra is a simple app for integrating the icelandic national registry into your django project",
    author = "Úlfur Kristjánsson",
    author_email = "ulfur@theawesometastic.com",
    url = "https://github.com/ulfur/django-thjodskra",
    packages = find_packages( ),
	include_package_data=True
)



#encoding: utf-8
from datetime import datetime

def l2d( d ):
	return datetime.strptime( d, '%d%m%Y' ) if d else None

def s2d( d ):
	return datetime.strptime( d, '%d%m%y' ) if d else None
	
property_map = [
	( 'einkenni', 			'type', 			 2, str ),	#Einkenni: FE - Einstaklingar, FF - Fyrirtæki
	( 'nafnnumer', 			'personal_id', 		 8, str ),	#Nafnnúmer
	( 'kennitala', 			'ssn', 				10, str ),	#Kennitala
	( 'fjolskyldunumer',	'family_id', 		10, str ),	#Fjölskyldunúmer
	( 'sort_code', 			'sort_code', 		31, str ),	#Röðunarsvæði fyrir nafn
	( 'logheimili', 		'domicile', 		12, str),	#Lögheimiliskóði
	( 'skip_1', 			'skip_1', 			 3, str ),	#Autt
	( 'kyn', 				'sex', 				 1, str ),	#Kyn
	( 'hjuskaparstada',		'marital_status',	 1, str ),	#Hjúskaparstaða
	( 'skip_2', 			'skip_2', 			 2, str ),	#Autt
	( 'bannmerking',		'blocked', 			 1, str ),	#Bannmerking
	( 'rikisfang', 			'nationality', 		 2, str ),	#Ríkisfang
	( 'faedingarstadur', 	'place_of_birth', 	 4, str ),	#Fæðingastaður
	( 'faedingardagur', 	'date_of_birth', 	 8, l2d ),	#Fæðingadagur
	( 'nafn', 				'name', 			31, str ),	#Nafn
	( 'kennitala_maka', 	'ssn_spouse', 		10, str ),	#Kennitala maka
	( 'logheimili_112', 	'domicile_112', 	12, str ),	#Lögheimiliskóði 1. Des s.l.
	( 'dags_hreyfing', 		'mod_date', 		 6, s2d ),	#Dagsetning hreyfingar
	( 'adsetur', 			'residence', 		12, str ),	#Aðsetur - lögheimiliskóði
	( 'logheimili_sidast',	'domicile_last', 	12, str ),	#Síðasta lögheimili á Íslandi
	( 'kennitala_umbods', 	'ssn_guardian', 	10, str ),	#Kennitala umboðsmanns
	( 'skip_3', 			'skip_3', 			13, str ),	#Autt
	( 'sokn', 				'parish', 			 3, str ),	#Sókn
	( 'postnumer', 			'postcode', 		 3, str ),	#Póstnúmer
	( 'heimilisfang_nf', 	'address', 			21, str ),	#Heimilisfang í nefnifalli
	( 'heimilisfang_thf', 	'address_thf', 		21, str ),	#Heimilisfang í þágufalli
	( 'skip_4', 			'skip_4', 			23, str ),	#Autt
	( 'afdrif',  			'status', 			 4, str ),	#Afdrif
	( 'dags_afdrif', 		'status_date', 		 6, s2d )	#Dagsetning afdrifa
]


'''
	Skýringar:
	
		Einkenni:
			FE - Einstaklingur
			FF - Fyrirtæki
			
		Kyn:
			1 - Karlmaður, 18 ára eða eldri
			2 - Kvenmaður, 18 ára eða eldri
			3 - Drengur, 17 ára eða yngri
			4 - Stúlka, 17 ára eða yngri
			
		Hjúskaparstaða:
			1 - Ógift
			3 - Gift eða staðfest samvist
			4 - Ekkill / Ekkja
			5 - Skilin að borð og sæng
			6 - Skilin að lögum
			7 - Hjón ekki í samvistum
			8 - Íslendingur í hjúskap með útlending
			9 - Hjúskaparstaða óupplýst
			0 - Íslendingur búsettur erlendis í hjúskap með útlendingi
			L - Íslendingur með lögheimili á Íslandi í hjúskap með óskráðum útlendingi
			
		Afdrif:
			AFMÐ - Afmáð 
			BRFD - Breytt kennitala og fæðingardagur  
			BRFL - Brottfelling (brottfluttur af landi, oftast útlendingar) 
			BRNF - Breytt nafn 
			BRNN - Breytt nafnnúmer 
			LÉST - látinn 
			Féla - Félagi slitið 
			Firm - félagi slitið; … 
			Fyri - afskráð; félagsslit ; félag lagt niður, hætt starfsemi; … 
			Hags - engin starfsemi; hætt; sameining; samruni; … 
			Hlut - félagsslit, samr; afskráð í hfskrá ; félagi skipt ; starfs.hætt;  
			Lögb - félagi slitið, þrotabú ; afmáð úr fi-skrá; þrotabú eiganda ;  
			LRHG - engin starfsemi; starfsemi hætt; fyrrv. eitthvað; ….. 
			SAMR - Samruni 
			SKIP - Skiptastjóri
			
'''
	
def parse_line_is( line ):
	out = {}
	for key, en, length, f in property_map:
		out[key] = f( line[:length].decode('iso-8859-1').encode('utf-8').strip() )
		line = line[length:]
	return out


def parse_line_en( line ):
	out = {}
	for isl, key, length, f in property_map:
		out[key] = f( line[:length].decode('iso-8859-1').encode('utf-8').strip() )
		line = line[length:]
	return out





#encoding: utf-8
import sys
from collections import deque
from datetime import datetime
from django.core.management.base import BaseCommand, CommandError
from ...models import Person

class Command(BaseCommand):

    args = u'<filename>'
    help = u'Imports people from þjóðskrá'

    def handle(self, *args, **options):

		fname = args[0]
		f = open( fname, 'r' )
		count = 1
		ttime = 0
		lines = [ l for l in f.readlines() ]
		total = len( lines )
		print '%i lines to process'%total
		
		f.seek( 0 )
		buff = deque()

		for l in lines:
			s = datetime.now()
			
			p, fresh = Person.from_string( l )
			if fresh:
				buff.append( p )
				count += 1
			
			if len(buff) >= 250:
				Person.objects.bulk_create( buff )
				buff = []
				
			d = datetime.now() - s
			ttime += d.total_seconds()
			avg = ttime/count
			
			sys.stdout.write( '\r Saved %7i people. Remain: %.2fs. Per second: %.1f' % ( count, avg*(total-count), 1/avg ) )
			sys.stdout.flush()
		if len(buff)>0:
			Person.objects.bulk_create(buff)
			
		print '\nDone'


from django.contrib import admin
from .models import Person

class PersonAdmin( admin.ModelAdmin ):
	list_display = ( 'name', 'ssn', 'address', 'postcode', 'blocked' )
	list_filter = ('sex',)
	search_fields = ('name', 'ssn')
	
admin.site.register( Person, PersonAdmin )